const mfs100Client = (function () {
    // Private variables
    // let defaultMfs100ClientUri = "http://192.168.0.4:8004/mfs100/"; // Default Non-Secure
    let keyFlag = "";
    let isGetSuccess = false;

    // Private functions
    function getHttpError(error) {
        if (error instanceof Response) {
            if (error.status === 0) {
                return 'Service Unavailable';
            } else if (error.status === 404) {
                return 'Requested page not found';
            } else if (error.status === 500) {
                return 'Internal Server Error';
            } else {
                return 'Unhandled Error';
            }
        } else {
            return "Network Error or Unhandled Exception: " + error.message;
        }
    }

    async function prepareScanner(ipAddress) {
        try {
            if (!isGetSuccess) {
                const resInfo = await getMFS100Client(ipAddress, "info");
                if (!resInfo.httpStatus) {
                    return false;
                } else {
                    isGetSuccess = true;
                }

                if (keyFlag && keyFlag.length > 0) {
                    const mfs100Request = {
                        "Key": keyFlag,
                    };
                    const jsonData = JSON.stringify(mfs100Request);
                    await postMFS100Client(ipAddress, "keyinfo", jsonData);
                }
            }
        } catch (e) {
            console.error("prepareScanner Error: ", e);
        }
        return true;
    }

    function getFalseRes() {
        return {httpStatus: false, error: "Error while calling service"};
    }


    async function postMFS100Client(ipAddress, method, jsonData) {
        try {
            let baseUrl = "http://" + ipAddress + ":8004/mfs100/";
            const response = await fetch(baseUrl + method, {
                method: "POST",
                headers: {
                    "Content-Type": "application/json; charset=utf-8",
                },
                body: jsonData,
            });

            if (response.ok) {
                const data = await response.json();
                return {httpStatus: true, data: data};
            } else {
                return {httpStatus: false, error: getHttpError(response)};
            }
        } catch (error) {
            return {httpStatus: false, error: getHttpError(error)};
        }
    }

    async function getMFS100Client(ipAddress, method) {
        try {
            let baseUrl = "http://" + ipAddress + ":8004/mfs100/";
            const response = await fetch(baseUrl + method, {
                method: "GET",
                headers: {
                    "Content-Type": "application/json; charset=utf-8",
                },
            });

            if (response.ok) {
                const data = await response.json();
                return {httpStatus: true, data: data};
            } else {
                return {httpStatus: false, error: getHttpError(response)};
            }
        } catch (error) {
            return {httpStatus: false, error: getHttpError(error)};
        }
    }

    // Public functions
    function getMFS100Info(ipAddress) {
        keyFlag = "";
        return getMFS100Client(ipAddress, "info");
    }

    function getMFS100KeyInfo(ipAddress, key) {
        keyFlag = key;
        if (!prepareScanner(ipAddress)) {
            return getFalseRes();
        }
        const mfs100Request = {
            "Key": key,
        };
        const jsonData = JSON.stringify(mfs100Request);
        return postMFS100Client(ipAddress, "keyinfo", jsonData);
    }

    async function captureFinger(ipAddress, scannerId, quality, timeout) {
        if (!await prepareScanner(ipAddress)) {
            var errorFingerprint = {
                "successful": false,
                "scannerId": scannerId,
                "remarks": "Couldn't contact device"
            }

            await inayaFingerprintDevice.postScannedImageToInaya(errorFingerprint);
            return
        }

        const mfs100Request = {
            "Quality": quality,
            "TimeOut": timeout,
        };

        const jsonData = JSON.stringify(mfs100Request);
        var result = await postMFS100Client(ipAddress, "capture", jsonData);

        var fingerprint;
        if (result.httpStatus) {
            if (result.data.ErrorCode == "0") {
                fingerprint = {
                    "successful": true,
                    "scannerId": scannerId,
                    "image": result.data.BitmapData,
                    "quality": result.data.Quality,
                    "template": result.data.IsoTemplate,
                }


            } else {
                fingerprint = {
                    "successful": false,
                    "scannerId": scannerId,
                    "remarks": result.data.ErrorDescription
                }
            }
        } else {
            fingerprint = {
                "successful": false,
                "scannerId": scannerId,
                "remarks": result.error
            }
        }

        await inayaFingerprintDevice.postScannedImageToInaya(fingerprint);
    }

    function verifyFinger(ipAddress, probFMR, galleryFMR) {
        if (!prepareScanner(ipAddress)) {
            return getFalseRes();
        }
        const mfs100Request = {
            "ProbTemplate": probFMR,
            "GalleryTemplate": galleryFMR,
            "BioType": "FMR",
        };
        const jsonData = JSON.stringify(mfs100Request);
        return postMFS100Client(ipAddress, "verify", jsonData);
    }

    function matchFinger(ipAddress, quality, timeout, galleryFMR) {
        if (!prepareScanner(ipAddress)) {
            return getFalseRes();
        }
        const mfs100Request = {
            "Quality": quality,
            "TimeOut": timeout,
            "GalleryTemplate": galleryFMR,
            "BioType": "FMR",
        };
        const jsonData = JSON.stringify(mfs100Request);
        return postMFS100Client(ipAddress, "match", jsonData);
    }

    function getPidData(ipAddress, biometricArray) {
        if (!prepareScanner(ipAddress)) {
            return getFalseRes();
        }
        const req = new MFS100Request(biometricArray);
        const jsonData = JSON.stringify(req);
        return postMFS100Client(ipAddress, "getpiddata", jsonData);
    }

    function getProtoPidData(ipAddress, biometricArray) {
        if (!prepareScanner(ipAddress)) {
            return getFalseRes();
        }
        const req = new MFS100Request(biometricArray);
        const jsonData = JSON.stringify(req);
        return postMFS100Client(ipAddress, "getppiddata", jsonData);
    }

    function getRbdData(ipAddress, biometricArray) {
        if (!prepareScanner(ipAddress)) {
            return getFalseRes();
        }
        const req = new MFS100Request(biometricArray);
        const jsonData = JSON.stringify(req);
        return postMFS100Client(ipAddress, "getrbddata", jsonData);
    }

    function getProtoRbdData(ipAddress, biometricArray) {
        if (!prepareScanner(ipAddress)) {
            return getFalseRes();
        }
        const req = new MFS100Request(biometricArray);
        const jsonData = JSON.stringify(req);
        return postMFS100Client(ipAddress, "getprbddata", jsonData);
    }

    // Classes
    function Biometric(bioType, biometricData, pos, nfiq, na) {
        this.BioType = bioType;
        this.BiometricData = biometricData;
        this.Pos = pos;
        this.Nfiq = nfiq;
        this.Na = na;
    }

    function MFS100Request(biometricArray) {
        this.Biometrics = biometricArray;
    }


    async function init() {
        var res = await getMFS100Info("127.0.0.1");
        if (res.httpStatus) {
            if (res.data.ErrorCode == "0") {
                inayaFingerprintDevice.registerDeviceOnInaya(
                    'MFS100',
                    res.data.DeviceInfo.LocalIP,
                    res.data.DeviceInfo.Make + "-" + res.data.DeviceInfo.Model + "-" + res.data.DeviceInfo.SerialNo);
            }
        } else {
            console.log("failed to init fingerprint device: " + res.error);
        }
    }

    init();

    // Public API
    return {
        captureFinger: captureFinger,

        getMFS100Info: getMFS100Info,
        getMFS100KeyInfo: getMFS100KeyInfo,
        verifyFinger: verifyFinger,
        matchFinger: matchFinger,
        getPidData: getPidData,
        getProtoPidData: getProtoPidData,
        getRbdData: getRbdData,
        getProtoRbdData: getProtoRbdData,
        Biometric: Biometric,
        MFS100Request: MFS100Request,
    };
})();